;;; kjd-mls.el --- Encode and Decode multiline strings.
;;
;; ~/share/emacs/pkg/kjd/kjd-mls.el ---
;;
;; $Id: kjd-mls.el,v 1.8 2003/11/07 18:27:20 harley Exp $
;;

;; Author:    Harley Gorrell <harley@panix.com>
;; URL:       http://www.mahalito.net/~harley/elisp/kjd-mls.el
;; License:   GPL v2
;; Keywords:  gcc multi line string

;;; Commentary:
;; * Gcc 3.3 does not like multiple line strings.
;;   The strings may be spliced together but must start and
;;   stop on the same line

;;; History:
;;  * 07-Aug-2003: Written for James.

;;; Code:

(defvar kjd-mls-text-start "\"\\n"
  "Text to add or remove at the start of a line.")
(defvar kjd-mls-text-end "\""
  "Text to add or remove at the end   of a line.")

;; Or if you like it the other way:
;; (setq kjd-mls-text-start "\"" kjd-mls-text-end "\\n\"")

(defvar kjd-mls-func-start-regexp "\\(printf\\|puts\\)+(" ;;"\\sw+("
  "A regexp to match a printing function.")
(defvar kjd-mls-func-end-regexp ");"
  "A regexp to match a end of a printing function.")

(defun kjd-mls-encode-region (r-s r-e)
  "Convert the region R-S to R-E into multi-line strings."
  (interactive "r")
  (save-restriction
    ;; dont work on text outside the region
    (narrow-to-region r-s r-e)
    ;; Do the first text-start
    (goto-char (point-min))
    (insert kjd-mls-text-start)
    ;; do the rest of the text-starts
    (while (re-search-forward "^" (point-max) t)
      (replace-match kjd-mls-text-start nil t)
      (end-of-line))
    ;; put in all but the last text-end
    (goto-char (point-min))
    (while (re-search-forward "$" (1- (point-max)) t)
      (replace-match kjd-mls-text-end nil t)
      ;; skip over eol
      (if (= (point) (point-max)) nil (forward-char 1)) )
    ;; put in the last text-end
    (goto-char (point-max))
    (insert kjd-mls-text-end)
    nil))

(defun kjd-mls-decode-region (r-s r-e)
  "Convert the region R-S to R-E from a multi-line string."
  (interactive "r")
  (let ((regexp-start (concat "^" (regexp-quote kjd-mls-text-start)    ))
        (regexp-end   (concat     (regexp-quote kjd-mls-text-end  ) "$")))
    (save-restriction
      ;; dont work on text outside the region
      (narrow-to-region r-s r-e)
      ;; Remove text-starts
      (goto-char (point-min))
      (while (re-search-forward regexp-start (point-max) t)
        (replace-match "" nil t))
      ;; remove text-ends
      (goto-char (point-min))
      (while (re-search-forward regexp-end (point-max) t)
        (replace-match "" nil t))
      nil)))

;; (global-set-key [f6] 'kjd-mls-encode-region)
;; (global-set-key [f7] 'kjd-mls-decode-region)

(defun kjd-mls-printf-region ()
  "Return the region of the text of the printf or nil."
  (save-excursion
    (let* ((pnt (point))
           r-s r-e p-min)
      ;; dont skip over an end.
      (goto-char pnt) 
      (setq p-min (or (re-search-backward kjd-mls-func-end-regexp (point-min) t)
                      (point-min) ))
      ;; find a start
      (goto-char pnt) 
      (setq r-s (re-search-backward kjd-mls-func-start-regexp p-min t))
      (if (not r-s)
        nil
        (setq r-s (match-end 0))
        ;; find an end
        (goto-char pnt)
        (setq r-e (re-search-forward kjd-mls-func-end-regexp (point-max) t))
        (if (not r-e)
            nil
          (setq r-e (match-beginning 0))
          (if (> r-s r-e) 
              nil
            (list r-s r-e) ))))))

(defun kjd-mls-encode-printf ()
  "Encode a multi-line-string within this printf."
  (interactive)
  (let ((region (kjd-mls-printf-region)))
    (if region
        (kjd-mls-encode-region (car region) (cadr region))
      (error "No printf region found"))))

(defun kjd-mls-decode-printf ()
  "Decode a multi-line-string within this printf."
  (interactive)
  (let ((region (kjd-mls-printf-region)))
    (if region
        (kjd-mls-decode-region (car region) (cadr region))
      (error "No printf region found"))))

;; (global-set-key [f6] 'kjd-mls-encode-printf)
;; (global-set-key [f7] 'kjd-mls-decode-printf)

(defun kjd-mls-smart-toggle ()
  (interactive)
  (let* ((region (kjd-mls-printf-region))
         (r-s (car region))
         (r-e (cadr region)))
    (when (not region)
      (error "No printf region found"))
    ;; found
    (goto-char r-s)
    (if (looking-at "\"")
        (kjd-mls-decode-region r-s r-e)
      (kjd-mls-encode-region r-s r-e))))
      
;; (global-set-key [f6] 'kjd-mls-smart-toggle)

(provide 'kjd-mls)

;;; kjd-mls.el ends here
